﻿var sortTableId = undefined;

// Default table sort options - should be called once DOM is loaded.
function initTableSort(tableId) {
    sortTableId = tableId;

    let storedSort = loadTableSort(tableId);

    $(`#${tableId}`).bootstrapTable({
        stickyHeader: true,
        sortable: true,
        sortName: storedSort?.sortName,
        sortOrder: storedSort?.sortOrder,
        onSort: tableOnSort
    });

    $('.clear-sort').on('click', { tableId: tableId }, clearSort);
}

// To Supplement the Bootstrap Table library
function sortDates(a, b) {
    let dateA = new Date(a);
    let dateB = new Date(b);
    if ((a.length === 0 && b.length > 0) || dateA < dateB) return -1;
    if ((a.length > 0 && b.length === 0) || dateA > dateB) return 1;
    return 0;
}

// Data is sorted by content, thus we need a special handler to sort links by their visible text.
function sortLinks(a, b) {
    a = a.replace('&nbsp;',' ').trim();
    b = b.replace('&nbsp;',' ').trim();
    if (a.length === 0 && b.length === 0) return 0;
    if (a.length === 0 && b.length > 0) return -1;
    if (a.length > 0 && b.length === 0) return 1;
    let linkRx = /<a[^>]+>([^<]+)<\/a>/i;
    let aMatch = a.match(linkRx);
    let bMatch = b.match(linkRx);
    let aText = aMatch.length > 1 ? aMatch[1] : '';
    let bText = bMatch.length > 1 ? bMatch[1] : '';
    return aText.localeCompare(bText);
}

// Load the persistent table sort from local storage
function loadTableSort(tableId) {
    let storedSort = sessionStorage.getItem(`${tableId}`);

    if (storedSort) {
        console.log(`Loaded Stored Sort: ${tableId}|${storedSort}`);
        let vals = storedSort.split(':');
        return { sortName: vals[0], sortOrder: vals[1] };
    }

    console.debug(`Stored Sort not found for table id: ${tableId}`);
}

// Common event handler for Bootstap Table onSort event.
// Hack note: requires sortTableId value specified on the page calling it to be used as the persistent storage key.
function tableOnSort(name, order) {
    if (!sortTableId) {
        console.error('sortTableId missing.  Cannot save table sort order.');
        return;
    }

    console.log(`Saving Updated Sort: ${sortTableId}|${name}:${order}`);
    sessionStorage.setItem(sortTableId, `${name}:${order}`);
}

// Click event for the clear sort button on pages with sortable tables
function clearSort(e) {
    // clear stored sort if able
    if (e.data.tableId) {
        console.log(`Removing Sort: ${e.data.tableId}`);
        sessionStorage.removeItem(e.data.tableId);
    }

    // Click the filter button if one exists, otherwise reload the page
    $filterButton = $('.filters #goBtn');
    if ($filterButton.length)
        $filterButton.click();
    else
        window.location.reload();
}
