﻿$.fn.selectpicker.Constructor.DEFAULTS.noneSelectedText = '';   // Remove text from multi-select boxes when nothing is selected
$.fn.selectpicker.Constructor.DEFAULTS.selectedTextFormat = 'count > 2';   // Show "counts" when more than 2 items are selected

$(function () {
    // Global Form Setup
    $('[data-type="date"]')
        .attr('placeholder', 'MM/DD/YYYY')
        .toArray()
        .forEach(function (field) {
            new Cleave(field, {
                date: true,
                datePattern: ['m', 'd', 'Y']
            });
        });

    $('[data-type="number"]')
        .toArray()
        .forEach(function (field) {
            new Cleave(field, {
                numeral: true,
                numeralThousandsGroupStyle: 'thousand'
            });
        });

    // Auto trim input fields
    $('input[type="text"]').change(function (e) {
        $(this).val($(this).val().trimEnd());
    });

    // Init tooltips everywhere
    $('[data-toggle="tooltip"]').tooltip({
        "offset": "0, 10px",
        "html": true
    });

    // Init special labels
    // TODO: Expand this to be more fluid
    $('.date-range-field').each(function (i, el) {
        // Add Modal Message box to page
        // TODO: Add this later, currently being done with and MVC Partial View.

        // Add Modal Message Icon
        $(getModalMessageIcon()).insertAfter(el);
    })

    // Wire up interfolio link objects for editable fields
    $('.interfolio-link').click(function (e) {
        e.preventDefault();
        var $button = $(this);
        openInterfolioLink($($button.data('target-selector')).val());
    });

    var formSubmitted = false;
    // Wire up form submissions with single-click check
    $(".submit-button").on('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        if (!formSubmitted) {
            formSubmitted = true;
            $(this).closest('form').submit();
        }
    });

    // Toggles visibility of a target element
    $('.toggle-button').click(function (e) {
        e.preventDefault();
        var $button = $(this);
        $($button.data('target-selector') + '[data-id="' + $button.data('target-id') + '"]').toggle();
    });

    // Capture all Filter form submissions and check min dates
    $("#goBtn").click(function (e) {
        e.preventDefault();
        var hasError = false;

        $('[data-type="date"]').each(function (i) {
            var dateRx = /\d\d\/\d\d\/(?:19|[2-9]\d)\d\d/gi;    // Date format which forces the year to be 1900 or newer.
            var val = $(this).val();
            if (val && !isNaN(Date.parse(val)) && !dateRx.test(val)) {
                showValidationError($(this).attr('name'), 'Date must be after 01/01/1900');
                hasError = true;
            }
        });

        if (!hasError) {
            $('#filters').submit();
        }
    });

    

    $(".delete-row-button").click(function (e) {
        e.preventDefault();

        if (!confirm("Are you sure you would like to delete this item?")) {
            return false;
        }
        var $row = $(this).closest('tr');
        var $btn = $(this);
        let id = $btn.data('id');
        let url = $btn.attr('href');

        if (!url) {
            alert('Could not load Delete URL');
            return false;
        }

        if (!id) {
            alert('Could not load Delete ID');
            return false;
        }

        $.post(url, { id: id })
            .done(function () {
                showMessage('Record deleted successfully.', msgTypes.SUCCESS);
                $row.remove();  // Remove the row from the table on successful deletion
            })
            .fail(function () {
                showMessage('Deletion Failed!', msgTypes.ERROR);
            });
    });
});

var interfolioRootUrl = 'https://facultysearch.interfolio.com/16712';

function captureEnter(selector, f) {
    $(selector).on("keypress", function (e) {
        if (e.which == 13) f();
    });
}

function getInterfolioUrl(posId, appId) {
    let url = '';

    if (posId && !isNaN(posId)) {
        url = `${interfolioRootUrl}/positions/${posId}`;

        if (appId && !isNaN(appId)) {
            url += `/applications/${appId}`;
        }
    }

    return url;
}

function getInterfolioLink(posId, appId) {
    let link = '';

    let url = getInterfolioUrl(posId, appId);

    if (url) {
        link = $('<a>').attr('href', url).text(`${appId ? appId : posId}`);
    }

    return link;
}

// Position ID is always required.  Potentially adding application ID for further drill down
function openInterfolioLink(posId, appId) {
    let url = getInterfolioUrl(posId, appId);

    if (url) {
        window.open(url, '_blank').focus();
    }
}

function submitFormOnChange(e) {
    $(e.target).closest('form').submit();
}

function formatJsonErrors(errors, useHtml) {
    let newline = useHtml ? '<br />' : '\n';
    if (errors.startsWith('{') || errors.startsWith('['))
        return errors.replace(/"":|"|\{|\}|\[|\]/gi, '').replace(/,/gi, newline);

    return errors;
}

// Replaces the options for a select list, denoted by the provided selector, 
// with the KVP object of new options. { "Text": "Value" }
function replaceOptions(selector, options) {
    var $el = $(selector);
    $el.empty();
    $.each(options, function (key, value) {
        $el.append($('<option>').val(value).text(key));
    });
}

// Will union the current value with the selected drop down items to ensure we don't get blanks.
// Can be re-used on select lists that have also been altered as it removes previously added elements.
// This currently runs case-sensitive, but hopefully it won't be an issue with lookup values.
function unionCurrentValue(selector, currentValue) {
    let $el = $(selector);
    if (!$el.length)
        return;

    $('option.added', $el).remove();    // Remove any options that may have been added previously

    if (!$('option[value="' + currentValue + '"]', $el).length)
        $('<option>').val(currentValue).text(currentValue).addClass('added').appendTo($el);
}

function clearFilters() {
    $('#filters input[type!="hidden"][type!="radio"], #filters select').val('');
    $('#filters input[type="checkbox"]').prop('checked', false);
    $('#filters select.selectpicker').selectpicker('render');
    $('#goBtn').click();
}

// Clears out validation errors in a modal popup identified by the provided selector
function clearModalValidationErrors(containerSelector) {
    $(containerSelector + ' .input-validation-error').removeClass('input-validation-error');
}

function showValidationError(fieldName, errorMessage) {
    //<span class="field-validation-valid" data-valmsg-for="Form.StatusDate" data-valmsg-replace="true"></span>
    //<span class="field-validation-error" data-valmsg-for="Form.InitiationDate" data-valmsg-replace="true">Date value must be equal to or newer than 01/01/1900.</span>
    // Highlight the main field itself
    $('[name="' + fieldName + '"]').addClass('input-validation-error');

    // Populate and show the error message
    $('[data-valmsg-for="' + fieldName + '"]').text(errorMessage).removeClass('field-validation-valid').addClass('field-validation-error');
}

function getLoadingImage() {
    return '<div class="loading-icon"><img src="/img/layout/loading.gif" class="loading-icon" />Loading...</div>';
}

function loadSelectListItems(targetSelector, items, includeBlank, selectedValue, blankText) {
    var $list = $(targetSelector);
    $('option', $list).remove();

    if (includeBlank) {
        let $blank = $('<option>').attr('value','');

        if (blankText) {
            $blank.text(blankText);
        }
        
        $blank.appendTo($list);
    }

    for (let i = 0; i < items.length; i++) {
        $('<option>').attr('value', items[i].value).text(items[i].text).appendTo($list);
    }

    if (selectedValue) {
        $list.val(selectedValue);
    } else {
        $('option', $list).first().attr('selected', 'selected');
    }
}

// Selects a value from the list if it exists
function activateSelectListItem(targetSelector, desiredValue) {
    if ($(`${targetSelector} option[value="${desiredValue}"]`).length)
        $(targetSelector).val(desiredValue);
}

// Error logging from JS methods
// Just post it and forget it, no need to worry about the return.
function log(level, message) {
    $.post('/api/log', { level: level, message: message });

    // Fetch isn't playing nicely with the controller binding
    //let data = new FormData();
    //data.append('level', level);
    //data.append('message', message);
    //data.append('message', message); 542pwd


    //const resp = await fetch('/api/log', {
    //    method: 'POST',
    //    body: data
    //});

    //if (!resp.ok)
    //    console.error('Failed to send log entry.');
}

function appendLeadingZero(num) {
    if (num < 9) {
        return '0' + num;
    }

    return num;
}

function formatDate(date) {
    // Don't forget to add one to the month because the Javascript creators are brain damaged
    return appendLeadingZero(date.getMonth()+1) + '/' + appendLeadingZero(date.getDate()) + '/' + date.getFullYear();
}

function formatNumber(num) {
    return num.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
}

function getFoyForDate(date) {
    if (date) {
        let foy = date.getFullYear();
        return date.getMonth() < 6 ? foy - 1 : foy;
    }

    return new Date().getFullYear();
}

function getAYDisplay(foy) {
    return foy + '-' + (foy + 1).toString().substring(2, 4);
}

// Unit placement is -1 - to the left, 0 - not shown, 1 - to the right
function currencyFormat(num, unit = '$', unitPlacement = -1) {
    var ret = '';
    if (unitPlacement < 0) { ret += unit; }
    ret += num.toFixed(2).replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
    if (unitPlacement > 0) { ret += unit; }

    return ret;
}

function formatMoney(num, unit = '$', unitPlacement = -1) {
    if (num) {
        return (num*1).toFixed(0).replace(/\d(?=(\d{3})+\.)/g, '$&,');
    }
    return '';
}

// Validation
function validateRequired(val) {
    if (val && val !== '') {
        return true;
    }

    return false;
}

function validateNumeric(val) {
    if (val && !isNaN(val)) {
        return true;
    }

    return false;
}

function validateDate(val) {
    if (/^[01]?\d\/\d?\d\/\d{4}/.test(val)) {
        return true;
    }

    return false;
}